import * as mongodb from "mongodb";
import { EntityMapping } from "./mapping/entityMapping";
import { ResultCallback } from "./core/callback";
import { InternalSession } from "./session";
import { ChangeTrackingType } from "./mapping/mappingModel";
import { IdentityGenerator } from "./config/configuration";
import { Batch } from "./batch";
import { Callback } from "./core/callback";
import { QueryDefinition } from "./query/queryDefinition";
import { QueryDocument } from "./query/queryBuilder";
import { Observer } from "./observer";
import { Property } from "./mapping/property";
export interface Persister {
    changeTracking: ChangeTrackingType;
    identity: IdentityGenerator;
    areDocumentsEqual(context: DirtyCheckContext, entity: Object, originalDocument: Object): boolean;
    dirtyCheck(batch: Batch, entity: Object, originalDocument: Object): Object;
    addInsert(batch: Batch, entity: Object): Object;
    addRemove(batch: Batch, entity: Object): void;
    fetch(entity: Object, path: string, callback: Callback): void;
    fetchPropertyValue(entity: any, property: Property, callback: ResultCallback<any>): void;
    refresh(entity: Object, callback: ResultCallback<Object>): void;
    watch(value: any, observer: Observer): void;
    executeQuery(query: QueryDefinition, callback: ResultCallback<any>): void;
    findOneById(id: any, callback: ResultCallback<any>): void;
    findInverseOf(entity: Object, path: string, callback: ResultCallback<Object[]>): void;
    findOneInverseOf(entity: Object, path: string, callback: ResultCallback<Object>): void;
}
export interface DirtyCheckContext {
    error?: Error;
}
export declare class PersisterImpl implements Persister {
    changeTracking: ChangeTrackingType;
    identity: IdentityGenerator;
    private _versioned;
    private _findQueue;
    private _mapping;
    private _collection;
    private _session;
    private _criteriaBuilder;
    private _updateDocumentBuilder;
    private _traceEnabled;
    private _defaultFields;
    constructor(session: InternalSession, mapping: EntityMapping, collection: mongodb.Collection);
    dirtyCheck(batch: Batch, entity: Object, originalDocument: Object): Object;
    areDocumentsEqual(context: DirtyCheckContext, entity: Object, originalDocument: Object): boolean;
    private _dirtyCheck;
    addInsert(batch: Batch, entity: Object): Object;
    addRemove(batch: Batch, entity: any): void;
    refresh(entity: any, callback: ResultCallback<Object>): void;
    watch(value: any, observer: Observer): void;
    private _refreshFromDocument;
    fetch(entity: any, path: string, callback: Callback): void;
    fetchPropertyValue(entity: any, property: Property, callback: ResultCallback<any>): void;
    findInverseOf(entity: Object, path: string, callback: ResultCallback<any[]>): void;
    findOneInverseOf(entity: Object, path: string, callback: ResultCallback<any>): void;
    _prepareInverseQuery(entity: Object, path: string, callback: ResultCallback<any>): QueryDocument;
    findOneById(id: any, callback: ResultCallback<any>): void;
    findOne(criteria: QueryDocument, callback: ResultCallback<any>): void;
    private _findOne;
    findAll(criteria: QueryDocument, callback: ResultCallback<any[]>): void;
    private _findAll;
    executeQuery(query: QueryDefinition, callback: ResultCallback<any>): void;
    private _executeQuery;
    private _createTraceableCallback;
    private _prepareOrderDocument;
    private _findEach;
    private _findEachSeries;
    private _findCursor;
    private _prepareFind;
    private _findOneAndModify;
    private _removeOne;
    private _removeAll;
    private _updateOne;
    private _updateAll;
    private _distinct;
    private _count;
    private _fetchOne;
    private _fetchAll;
    private _fetchIterator;
    private _loadOne;
    private _getCommand;
}
export interface Cursor<T> {
    next(callback: ResultCallback<T>): void;
    close(callback?: Callback): void;
}
