/// <reference types="node" />
import { EventEmitter } from "events";
import { Callback } from "./core/callback";
import { Constructor } from "./index";
import { ResultCallback } from "./core/callback";
import { InternalSessionFactory, SessionFactory } from "./sessionFactory";
import { Persister } from "./persister";
import { EntityMapping } from "./mapping/entityMapping";
import { QueryBuilder, FindOneQuery } from "./query/queryBuilder";
import { QueryDefinition } from "./query/queryDefinition";
export interface Session {
    factory: SessionFactory;
    save(obj: Object, callback?: Callback): void;
    remove(obj: Object, callback?: Callback): void;
    detach(obj: Object, callback?: Callback): void;
    refresh(obj: Object, callback: Callback): void;
    flush(callback?: Callback): void;
    clear(callback?: Callback): void;
    find<T>(ctr: Constructor<T>, id: any, callback?: ResultCallback<T>): FindOneQuery<T>;
    exists<T>(ctr: Constructor<T>, id: any, callback: ResultCallback<boolean>): void;
    getReference<T>(ctr: Constructor<T>, id: any): T;
    fetch<T>(obj: T, callback?: ResultCallback<T>): void;
    fetch<T>(obj: T, path: string | string[], callback?: ResultCallback<T>): void;
    query<T>(ctr: Constructor<T>): QueryBuilder<T>;
    wait(callback?: Callback): void;
    close(callback?: Callback): void;
    contains(obj: Object): boolean;
    toDocument(obj: Object, callback?: ResultCallback<Object>): Object;
    isDirty(obj: Object, callback?: ResultCallback<boolean>): boolean;
    getVersion(obj: Object): number;
    getEntities(): Object[];
    on(event: string, listener: Function): EventEmitter;
}
export interface InternalSession extends Session {
    factory: InternalSessionFactory;
    getObject(id: any): any;
    registerManaged(persister: Persister, entity: Object, document: any): void;
    notifyRemoved(entity: Object): void;
    getPersister(mapping: EntityMapping): Persister;
    getReferenceInternal(mapping: EntityMapping, id: any): any;
    fetchInternal(entity: Object, paths: string[], callback: ResultCallback<any>): void;
    executeQuery(query: QueryDefinition, callback: ResultCallback<any>): void;
}
export declare class SessionImpl extends EventEmitter implements InternalSession {
    factory: InternalSessionFactory;
    private _persisters;
    private _objectLinksById;
    private _objectLinks;
    private _scheduleHead;
    private _scheduleTail;
    private _queue;
    private _traceEnabled;
    constructor(factory: InternalSessionFactory);
    save(obj: any, callback?: Callback): void;
    remove(obj: any, callback?: Callback): void;
    refresh(obj: any, callback?: Callback): void;
    detach(obj: any, callback?: Callback): void;
    clear(callback?: Callback): void;
    flush(callback?: Callback): void;
    wait(callback?: Callback): void;
    find<T>(ctr: Constructor<T>, id: any, callback?: ResultCallback<T>): FindOneQuery<T>;
    exists<T>(ctr: Constructor<T>, id: any, callback: ResultCallback<boolean>): void;
    fetch<T>(obj: T, pathsOrCallback: any, callback?: ResultCallback<T>): void;
    close(callback?: Callback): void;
    executeQuery(query: QueryDefinition, callback: ResultCallback<any>): void;
    contains(obj: any): boolean;
    getEntities(): Object[];
    getReference<T>(ctr: Constructor<T>, id: any): T;
    toDocument(obj: Object, callback?: ResultCallback<Object>): Object;
    isDirty(obj: Object, callback?: ResultCallback<boolean>): boolean;
    getVersion(obj: Object): number;
    getReferenceInternal(mapping: EntityMapping, id: any): any;
    getObject(id: any): any;
    registerManaged(persister: Persister, entity: any, document: any): void;
    private _trackChanges;
    private _stopWatching;
    getPersister(mapping: EntityMapping): Persister;
    query<T>(ctr: Constructor<T>): QueryBuilder<T>;
    private _execute;
    private _save;
    private _saveEntities;
    private _makeDirty;
    private _clearDirty;
    notifyRemoved(entity: Object): void;
    private _remove;
    private _removeEntities;
    private _removeEntity;
    private _detach;
    private _detachEntities;
    private _refresh;
    private _refreshEntities;
    private _flush;
    private _buildBatch;
    private _batchCompleted;
    private _logFlushStats;
    private _close;
    fetchInternal(obj: any, paths: string[], callback: ResultCallback<any>): void;
    private _fetchPaths;
    private _clear;
    private _getObjectLinks;
    private _linkObject;
    private _unlinkObject;
    private _cleanupUnlinkedObject;
    private _scheduleOperation;
    private _clearScheduledOperation;
    private _clearIdentifier;
    private _findReferencedEntities;
    private _walk;
    private _createTaskQueue;
}
